import os.path as pth
#import matplotlib.pyplot as plt
import matplotlib as mpl
from matplotlib.style.core import read_style_directory, update_nested_dict, \
    available, library

# check if this finds the resources when installing the module using pip.
scistyles_dir = [pth.abspath(pth.join(pth.dirname(__file__), "stylelib"))]

def list_styles(style_dirs: list=None) -> dict:
    """List the styles available directory `styles_dirs`.
    """
    style_dirs = scistyles_dir if style_dirs is None else style_dirs
    styles = dict()
    for stylelib_path in style_dirs:
        styles.update(read_style_directory(stylelib_path))
    return styles


def update_library(
    style_dirs: list=None, library=None, available: list=None,
    ):
    """Load styles available in directory.
    
    Parameters
    ----------
    """
    style_dirs = scistyles_dir if style_dirs is None else style_dirs
    library = mpl.style.library if library is None else library
    available = mpl.style.library if available is None else available
    update_nested_dict(library, list_styles(style_dirs))
    available[:] = sorted(library.keys())
    return library

update_library(scistyles_dir, library, available)



# investigating how the library of styles is built.
# Maybe we should just extend it directly
# the debugger has problem running matplotlib...

import matplotlib.pyplot as plt
from pprint import pprint
from copy import deepcopy
import os.path as pth

# objects to investigate:
# avail
avail = plt.style.available
print("- "*30)
print("avail", "type", type(avail)) # list
pprint(avail)

# library
print("- "*30)
print("library", "type", type(plt.style.library)) # a dict subtype
pprint(plt.style.library.keys())

# library
print("- "*30)
print("USER_LIBRARY_PATHS", "USER_LIBRARY_PATHS", type(plt.style.core.USER_LIBRARY_PATHS)) # a dict subtype
pprint(plt.style.core.USER_LIBRARY_PATHS)

# we can probably just append to library.
# there are a bunch of tools to help:
# read_style_directory
# update_user_library #  Do something like this.

# check if we can append.

test = deepcopy(plt.style.library["fast"])

print(type(test))
test['path.simplify_threshold']=0.8

plt.style.library["notsofast"] = test

plt.style.use("notsofast")
assert plt.rcParams['path.simplify_threshold'] == 0.8 

scistyle_path = pth.abspath(pth.join(pth.dirname(__file__), "..", "stylelib"))
print(scistyle_path)
new_ = {"scistyles-"+k: v for
        k, v in plt.style.core.read_style_directory(scistyle_path).items()
}
plt.style.core.update_nested_dict(plt.style.library, new_)

assert "scistyles-ACS_nano" in plt.style.library
# good. We have path to work with
# 1. organize a directory of styles
# 2. reproduce something like 'update_user_library', updating the library
# 3. ???
# 4. profit




import pytest
import os.path as pth
from glob import glob
from itertools import chain
from scistyles import scistyles_dir, list_styles#, update_library
from matplotlib.style.core import library, available
from matplotlib.style import use

def test_files():
    # check that there is at least one style file
    files = list(*chain([glob(d+"/*.mplstyle") for d in scistyles_dir]))
    assert len(files) > 0

def test_list_styles():
    files = list(*chain([glob(d+"/*.mplstyle") for d in scistyles_dir]))
    listed = list_styles()
    assert len(files) == len(listed) # for now...

def test_autoupdate():
    # check all our styles got added to the list
    for stylename in list_styles():
        assert stylename in library

def test_available():
    # check this was synced into `available`
    for stylename in list_styles():
        assert stylename in available

def test_use():
    # check we can use the style
    for stylename in list_styles():
        use(stylename)
    assert True
